package Config::Tree::Node::Section;
use parent 'Config::Tree::Node';
use strict;
use warnings;
use Carp;
use Config::Tree::Node::Null;

sub new {
    my $class = shift;
    my $self = $class->SUPER::new(@_);
    $self->{_subtree} = {};
    return $self;
}

sub subtree {
    my $self = shift;
    if (my $key = shift) {
	if (my $val = shift) {
	    $self->{_subtree}{$key} = $val;
	}
	return $self->{_subtree}{$key};
    }
    return $self->{_subtree};
}

sub keys {
    my $self = shift;
    return keys %{$self->{_subtree}};
}

sub has_key {
    my ($self, $key) = @_;
    return $self->subtree($key);
}

sub delete {
    my ($self, $key) = @_;
    delete $self->{_subtree}{$key};
}

sub merge {
    my ($self, $other) = @_;
    while (my ($k, $v) = each %{$other->subtree}) {
	if (my $old = $self->subtree($k)) {
	    if ($old->is_section) {
		$old->merge($v);
	    } elsif (ref($old->value) eq 'ARRAY') {
		push @{$old->value}, $v->value;
		$old->locus->add_locus($v->locus);
	    } else {
		$old->value($v->value);
	    }
	} else {
	    $self->subtree($k => $old->clone);
	}
	$self->locus->add_locus($v->locus);
    }
}

=head2 $h = $cfg->as_hash

=head2 $h = $cfg->as_hash($map)    

Returns parse tree converted to a hash reference. If B<$map> is supplied,
it must be a reference to a function. For each I<$key>/I<$value>
pair, this function will be called as:

    ($newkey, $newvalue) = &{$map}($what, $key, $value)

where B<$what> is C<section> or C<value>, depending on the type of the
hash entry being processed. Upon successful return, B<$newvalue> will be
inserted in the hash slot for the key B<$newkey>.

If B<$what> is C<section>, B<$value> is always a reference to an empty
hash (since the parse tree is traversed in pre-order fashion). In that
case, the B<$map> function is supposed to do whatever initialization that
is necessary for the new subtree and return as B<$newvalue> either B<$value>
itself, or a reference to a hash available inside the B<$value>. For
example:

    sub map {
        my ($what, $name, $val) = @_;
        if ($name eq 'section') {
            $val->{section} = {};
            $val = $val->{section};
        }
        ($name, $val);
    }
    
=cut

sub as_hash {
    my $self = shift;
    my $map = shift // sub { shift; @_ };
    my $hroot = {};
    my @ar;
    
    push @ar, [ '', $self, $hroot ];
    while (my $elt = shift @ar) {
	if ($elt->[1]->is_section) {
	    my $hr0 = {};
	    my ($name, $hr) = &{$map}('section', $elt->[0], $hr0);
	    $elt->[2]{$name} = $hr0;
	    while (my ($kw, $val) = each %{$elt->[1]->subtree}) {
		push @ar, [ $kw, $val, $hr ];
	    }
	} else {
	    my ($name, $value) = &{$map}('value', $elt->[0], scalar($elt->[1]->value));
	    $elt->[2]{$name} = $value;
	}
    }
    return $hroot->{''};
}

sub as_string { '(section)' }

our $AUTOLOAD;

sub AUTOLOAD {
    my $self = shift;
    my $key = $AUTOLOAD;
    $key =~ s/.*:://;
    if ($key =~ s/^([A-Z])(.*)/\l$1$2/) {
	$key =~ s/__/-/g;
	return $self->subtree($self->{_ci} ? lc($key) : $key)
	       // new Config::Tree::Node::Null;
    }
    confess "Can't locate method $AUTOLOAD";
}

1;
